@@ -15,7 +15,7 @@ class acbuf
 {
     public:
         inline acbuf() : r(0), w(0), m_nCapacity(0), m_buf(NULL) {};
-    	inline ~acbuf() { free(m_buf); m_buf=NULL; }
+    	virtual ~acbuf() { free(m_buf); m_buf=NULL; }
     	inline bool empty() const { return w==r;}
     	//! Count of the data inside
         inline unsigned int size() const { return w-r;}
@@ -62,12 +62,13 @@ class acbuf
 
 
     protected:
-
         size_t r, w, m_nCapacity; // read/write positions, size
         char *m_buf;
     	acbuf(const acbuf&); // don't copy me
     	acbuf& operator=(const acbuf&);
 
+//        friend class maintenance::tFmtSendObj;
+
 };
 
 /* This is a light-weight and less cumbersome implementation of ostringstream.
@@ -81,7 +82,7 @@ public:
 // map char array to buffer pointer and size
 #define _SZ2PS(x) x, (sizeof(x)-1)
 	inline tSS & operator<<(const char *val) { return add(val); }
-	inline tSS & operator<<(const MYSTD::string& val) { return add(val); };
+	inline tSS & operator<<(const mstring& val) { return add(val); };
 	inline tSS & operator<<(const acbuf& val) { return add(val.rptr(), val.size()); };
 
 #define __tss_nbrfmt(x, h, y) { add(fmtbuf, sprintf(fmtbuf, m_fmtmode == hex ? h : x, y)); return *this; }
@@ -95,23 +96,25 @@ public:
     enum fmtflags {	hex, dec };
     inline tSS & operator<<(fmtflags mode) { m_fmtmode=mode; return *this;}
 
-    operator MYSTD::string() const { return MYSTD::string(rptr(), size()); }
+    operator mstring() const { return mstring(rptr(), size()); }
     inline size_t length() const { return size();}
     inline const char * data() const { return rptr();}
 
     inline tSS() : m_fmtmode(dec){}
     inline tSS(size_t sz) : m_fmtmode(dec) { init(sz); }
     inline tSS(const tSS &src) : m_fmtmode(src.m_fmtmode) { add(src.data(), src.size()); }
-    inline tSS & operator<<(const char val) { return add(&val, 1); }
+    tSS & addEscaped(const char *fmt);
+    inline tSS & operator<<(const char c) { reserve(size()+1); *(wptr())=c; got(1); return *this;}
+
 
 protected:
     char fmtbuf[22];
 	fmtflags m_fmtmode;
-	inline void reserve(size_t minCapa) { minCapa++; if(m_nCapacity>=minCapa) return;
+	inline void reserve(size_t minCapa) { minCapa+=(r+1); if(m_nCapacity>=minCapa) return;
 	char *p=(char*)realloc(m_buf, MYSTD::max(m_nCapacity, minCapa*2));
 	if(!p) throw std::bad_alloc(); m_nCapacity=minCapa*2; m_buf=p; }
 
-	inline tSS & add(const MYSTD::string& val) { return add(val.data(), val.size());}
+	inline tSS & add(const mstring& val) { return add(val.data(), val.size());}
 	inline tSS & appDosNL() { return add("\r\n", 2);}
 	inline tSS & add(const char *data, size_t len)
 	{ reserve(size()+len); memcpy(wptr(), data, len); got(len); return *this;}
