@@ -61,7 +61,7 @@ namespace aclog
       mstring GetStatReport();
       
       struct errnoFmter : public mstring {
-    	  errnoFmter();
+    	  errnoFmter(const char *prefix=NULL);
       };
 }
 
