@@ -3,6 +3,7 @@
 #define __DEBUG_H__
 
 #include "acfg.h"
+#include "aclogger.h"
 
 #ifndef SPAMLEVEL
 #define SPAMLEVEL 10
@@ -11,8 +12,6 @@
 #ifdef DEBUG
 
 #include <meta.h>
-
-#include "aclogger.h"
 #include <assert.h>
 #define ASSERT(x) { if(acfg::debug > SPAMLEVEL*2) assert(x); }
 
@@ -45,14 +44,13 @@
 #else
 
 #include <iostream>
-#include <sstream>
 
 #define ldbgvl(v, x)
 #define dbglvl(v)
 #define ldbg(x)
 #define dbgline
 #define ASSERT(x)
-#define USRDBG(v, x) { if((v)<=acfg::debug) {  MYSTD::ostringstream __oss; __oss << x; aclog::err(__oss.str());  } }
+#define USRDBG(v, x) { if((v)<=acfg::debug) aclog::err( tSS()<<x ); }
 #define LOG(x)
 #define LOGSTART(x)
 #define LOGSTART2(x,y)
