@@ -10,7 +10,7 @@
 //#include <netinet/in.h>
 //#include <netdb.h>
 
-#include "tcpconnect.h"
+//#include "tcpconnect.h"
 
 #include "lockable.h"
 #include "fileitem.h"
@@ -20,45 +20,48 @@
 class tDlJob;
 
 
-class dlcon : public lockable, tcpconnect
+class dlcon : public lockable
 { 
     public:
         dlcon(bool bManualExecution, mstring *xff=NULL);
         ~dlcon();
-                
+
         void WorkLoop();
         
         void SignalStop();
 
 	    void AddJob(tFileItemPtr m_pItem, tHttpUrl hi);
         void AddJob(tFileItemPtr m_pItem, 
-        		acfg::tHostiVec *pBackends, const mstring & sPatSuffix);
+        		acfg::tRepoData *pBackends, const mstring & sPatSuffix);
     	void EnqJob(tDlJob *);
 
         mstring m_sXForwardedFor;
         
     private:
+
+    	//not to be copied
+    	dlcon & operator=(const dlcon&);
+    	dlcon(const dlcon&);
     	
     	friend class tDlJob;
     	
-
     	MYSTD::list<tDlJob*> m_qToReceive;
     	int m_wakepipe[2];
-    	
-    	//mstring m_sConnectedHost; // used to remember to which host we are connected
 
     	acbuf m_InBuf;
     	
     	// flags and local copies for input parsing
     	/// remember being attached to an fitem
 
-    	bool m_bSingleRun;
-    	    
+    	bool m_bStopWhenIdle;
+
+    	/// shared blacklist for bad hosts
     	MYSTD::set<mstring> m_MirrorHostBlacklist;
 
-    	//not to be copied
-    	dlcon & operator=(const dlcon&);
-    	dlcon(const dlcon&);
+    	//! will establish the connection, keep its object pointer
+    	//! in m_pConnection and return its file descriptor (negative on failure)
+//    	int Connect(cmstring & sHostname, mstring &sErrOut);
+
 };
 
 typedef MYSTD::list<tDlJob*>::iterator dljIter; 
