@@ -12,7 +12,7 @@ class tDlJob
 {
 public:
 	
-	bool bSuggestReconnect, bRequestPrepared;
+	bool bSuggestReconnect, bRequestEnqueued;
 	
 	inline bool HasBrokenStorage()
 	{
@@ -33,18 +33,18 @@ public:
 
 	tDlJob(dlcon *m, tFileItemPtr pFi, const mstring & sHost,
 			const mstring & sPath);
-	tDlJob(dlcon *m, tFileItemPtr pFi, acfg::tHostiVec * pBackends,
+	tDlJob(dlcon *m, tFileItemPtr pFi, acfg::tRepoData * pBackends,
 			const mstring & sPath);
 	virtual ~tDlJob();
 	
 
-	void MarkFailed(const mstring &sError);
+	void UnregDownloader(const mstring &sError);
     void BlacklistBackend();
 	
 	bool FindConfig();
 	
 	// needs connectedHost, blacklist, output buffer from the parent, proxy mode?
-	bool AppendRequest(mstring sForThisHostOnly, tSS &head);
+	bool AppendRequest(mstring sForThisHostOnly, tSS &head, const tHttpUrl * pProxy);
 
 	inline bool HasStarted() { return m_DlState>STATE_GETHEADER; }
 
@@ -55,7 +55,7 @@ public:
     mstring RemoteUri();
     mstring RemotePath();
     
-    acfg::tHostiVec::IHookHandler * GetConnStateObserver();
+    acfg::tRepoData::IHookHandler * GetConnStateObserver();
 
 protected:
 	
@@ -75,7 +75,7 @@ protected:
 		,STATE_FINISHJOB
 	} tDlState;
 
-	acfg::tHostiVec * m_pHostiVec;
+	acfg::tRepoData * m_pHostiVec;
 	tFileItemPtr m_pStorage;
 	dlcon *m_parent;
 	
