@@ -81,7 +81,7 @@ class tCacheMan :
 
 public:
 	tCacheMan(int);
-	~tCacheMan();
+	virtual ~tCacheMan();
 
 protected:
 	enum enumIndexType
@@ -151,7 +151,8 @@ protected:
 	time_t m_nTimeNow;
 
 	bool m_bErrAbort, m_bVerbose, m_bForceDownload;
-	bool m_bScanInternals, m_bByPath, m_bByChecksum, m_bSkipHeaderChecks, m_bNeedsStrictPathsHere;
+	bool m_bScanInternals, m_bByPath, m_bByChecksum, m_bSkipHeaderChecks;
+	bool m_bNeedsStrictPathsHere, m_bTruncateDamaged;
 	int m_nErrorCount;
 
 	enumIndexType GuessIndexType(const mstring &sPath);
