@@ -35,8 +35,6 @@ class job {
 	   	STATE_FINISHJOB
 	   } eJobState;
 
-       MYSTD::string lpath, checksum; // relative local path and checksum, written by resolver
-
       job(header *h, con *pParent);
       ~job();
       //  __attribute__((externally_visible))  
@@ -59,11 +57,11 @@ class job {
       
       eJobState m_state, m_backstate;
       
-      //MYSTD::string m_sErrorMsgBuffer;
+      //mstring m_sErrorMsgBuffer;
       tSS m_sendbuf;
-      MYSTD::string m_sFileLoc; // local_relative_path_to_file
-      MYSTD::string m_sMaintCmd;
-      MYSTD::string m_sOrigUrl; // local SAFE copy of the originating source
+      mstring m_sFileLoc; // local_relative_path_to_file
+      mstring m_sMaintCmd;
+      mstring m_sOrigUrl; // local SAFE copy of the originating source
       
       header *m_pReqHead; // copy of users requests header
       
@@ -78,10 +76,10 @@ class job {
       job & operator=(const job&);
 
       const char * BuildAndEnqueHeader(const FiStatus &fistate, const off_t &nGooddataSize, header& respHead);
-      FiStatus _SwitchToPtItem(const MYSTD::string &fileLoc);
+      FiStatus _SwitchToPtItem(const mstring &fileLoc);
       void SetErrorResponse(const char * errorLine, int nMinUserLogLevel=SPAMLEVEL);
-      void HandleLocalDownload(const MYSTD::string &visPath,
-    			const MYSTD::string &fsBase, const MYSTD::string &fsSubpath);
+      void HandleLocalDownload(const mstring &visPath,
+    			const mstring &fsBase, const mstring &fsSubpath);
 };
 
 #endif
