@@ -113,11 +113,12 @@ typedef multimap<tHttpUrl,const string*, fct_lt_host> tMapUrl2StringPtr;
 typedef tMapUrl2StringPtr::iterator tUrl2RepIter;
 tMapUrl2StringPtr mapUrl2pVname;
 
-typedef map<const string, tHostiVec> tMapString2Hostivec;
-tMapString2Hostivec mapRepName2Backends;
+typedef map<const string, tRepoData> tRepMap;
+tRepMap repoparms;
 
-string * _GetStringPtr(const string &key)  {
-	for(unsigned int i=0; i<_countof(n2sTbl); i++) {
+string * _GetStringPtr(const string &key) {
+	for(unsigned int i=0; i<_countof(n2sTbl); i++)
+	{
 		if(0==strcasecmp(key.c_str(), n2sTbl[i].name))
 		{
 			if(n2sTbl[i].warn)
@@ -129,8 +130,9 @@ string * _GetStringPtr(const string &key)  {
 	return NULL;
 }
 
-int * _GetIntPtr(const string &key, int &base)  {
-	for(unsigned int i=0; i<_countof(n2iTbl); i++) {
+int * _GetIntPtr(const string &key, int &base) {
+	for(unsigned int i=0; i<_countof(n2iTbl); i++)
+	{
 		if(0==strcasecmp(key.c_str(), n2iTbl[i].name))
 		{
 			if(n2iTbl[i].warn)
@@ -176,21 +178,15 @@ inline bool _ReadMainConfiguration(const string & szFilename)
 	return true;
 }
 
-const string * _CheckBEentryGetNamePtr(const string & sRepName)
+inline const string * _CheckBEentryGetNamePtr(const string & sRepName)
 {
+	// some memory saving using STL containers properties:
 	// needs a reliably stored string for the pointer. Backend description may not exist,
 	// then create a dummy one with no contents and point at this string. Iterators on map are
 	// guaranteed to be stable so point at its key variable.
-	tMapString2Hostivec::iterator
-			itHostiVec = mapRepName2Backends.find(sRepName);
-	if (itHostiVec == mapRepName2Backends.end())
-	{
-		mapRepName2Backends[sRepName]=tHostiVec();
-		itHostiVec=mapRepName2Backends.find(sRepName); // must refer to that string on the heap
-		if (debug>4)
-			cout << "created empty backend entry for "<< sRepName <<endl;
-	}
-	return & itHostiVec->first;
+
+	pair<tRepMap::iterator,bool> addRes = repoparms.insert(make_pair(sRepName, tRepoData()));
+	return & addRes.first->first;
 }
 
 inline void _AddRemapInfo(bool bAsBackend, const string & token,
@@ -204,7 +200,7 @@ inline void _AddRemapInfo(bool bAsBackend, const string & token,
 		_FixPostPreSlashes(url.sPath);
 		
 		if (bAsBackend)
-			mapRepName2Backends[repname].push_back(url);
+			repoparms[repname].push_back(url);
 		else
 			mapUrl2pVname.insert(pair<tHttpUrl,const string*>(
 					url, _CheckBEentryGetNamePtr(repname)));
@@ -257,7 +253,7 @@ inline bool ParseOptionLine(const string &sLine, string &key, string &val, bool
 }
 
 
-struct tHookHandler: public tHostiVec::IHookHandler, public lockable
+struct tHookHandler: public tRepoData::IHookHandler, public lockable
 {
 	string cmdRel, cmdCon;
 	time_t downDuration, downTimeNext;
@@ -326,7 +322,7 @@ void _AddHooksFile(cmstring& vname)
 				hs.downDuration = n;
 		}
 	}
-	mapRepName2Backends[vname].m_pHooks = &hs;
+	repoparms[vname].m_pHooks = &hs;
 
 }
 
@@ -530,17 +526,22 @@ bool SetOption(const string &sLine, bool bQuiet)
 				cerr << "Found invalid entry, ignoring " << key << ": " << value <<endl;
 			return false;
 		}
-		bool bIsBackend=false;
+		enum xtype { prefix, backend, flags } type = prefix;
 		for(UINT i=0; i<tokens.size(); i++)
 		{
 			if(tokens[i]==";")
 			{
-				bIsBackend=true;
+				type=xtype(int(type)+1);
 				continue;
 			}
 			if(startsWithSz(tokens[i], "#"))
 				break;
-			_AddRemapInfo(bIsBackend, tokens[i], vname);
+			if(flags == type)
+			{
+#warning do something with the stuff
+			}
+			else
+				_AddRemapInfo(backend == type, tokens[i], vname);
 		}
 		_AddHooksFile(vname);
 	}
@@ -559,6 +560,7 @@ const string * GetRepNameAndPathResidual(const tHttpUrl & in, string & sRetPathR
 {
 	sRetPathResidual.clear();
 	
+	// get all the URLs matching THE HOSTNAME
 	pair<tUrl2RepIter,tUrl2RepIter> range=mapUrl2pVname.equal_range(in);
 	if(range.first==range.second)
 		return NULL;
@@ -566,6 +568,7 @@ const string * GetRepNameAndPathResidual(const tHttpUrl & in, string & sRetPathR
 	tStrPos bestMatchLen(0);
 	string const * psBestHit(NULL);
 		
+	// now find the longest directory part which is the suffix of requested URL's path
 	for (tUrl2RepIter & it=range.first; it!=range.second; it++)
 	{
 		// rewrite rule path must be a real prefix
@@ -587,12 +590,12 @@ const string * GetRepNameAndPathResidual(const tHttpUrl & in, string & sRetPathR
 	
 }
 
-tHostiVec * GetBackendVec(const string * vname)
+tRepoData * GetBackendVec(const string * vname)
 {
 	if(!vname)
 		return NULL;
-	tMapString2Hostivec::iterator it=mapRepName2Backends.find(*vname);
-	if(it==mapRepName2Backends.end() || it->second.empty())
+	tRepMap::iterator it=repoparms.find(*vname);
+	if(it==repoparms.end() || it->second.empty())
 		return NULL;
 	return & it->second;
 }
@@ -638,7 +641,7 @@ void _ReadBackendsFiles(const string & sFile, const string &sRepName)
 			cerr << "Backend: " << sRepName << " <-- " << entry.ToURI() <<endl;
 #endif		
 
-			mapRepName2Backends[sRepName].push_back(entry);
+			repoparms[sRepName].push_back(entry);
 			nAddCount++;
 			entry.clear();
 		}
@@ -685,7 +688,7 @@ try_read_default:
 void ShutDown()
 {
 	mapUrl2pVname.clear();
-	mapRepName2Backends.clear();
+	repoparms.clear();
 }
 
 void _ReadRewriteFiles(const string & sFile, const string & sRepName)
@@ -789,7 +792,7 @@ void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 }
 
 
-tHostiVec::~tHostiVec()
+tRepoData::~tRepoData()
 {
 	if(m_pHooks)
 	{
@@ -962,8 +965,8 @@ void PostProcConfig()
 
 #if 0 //def DEBUG
 #warning adding hook control pins
-   for(tMapString2Hostivec::iterator it = mapRepName2Backends.begin();
-		   it!=mapRepName2Backends.end() ; ++it)
+   for(tMapString2Hostivec::iterator it = repoparms.begin();
+		   it!=repoparms.end() ; ++it)
    {
 	   tHookHandler *p = new tHookHandler(it->first);
 	   p->downDuration=10;
@@ -979,7 +982,7 @@ void PostProcConfig()
 	   for(tMapString2Hostivec::const_iterator it=mapRepName2Backends.begin();
 			   it!=mapRepName2Backends.end(); it++)
 	   {
-		   for(tHostiVec::const_iterator jit=it->second.begin();
+		   for(tRepoData::const_iterator jit=it->second.begin();
 				   jit != it->second.end(); jit++)
 		   {
 			   cout << jit->ToURI() <<endl;
@@ -998,8 +1001,7 @@ void PostProcConfig()
 time_t ExecutePostponed()
 {
 	time_t ret(cleaner::never), now(time(0));
-	for (tMapString2Hostivec::iterator it = mapRepName2Backends.begin(); it
-			!= mapRepName2Backends.end(); ++it)
+	for (tRepMap::iterator it = repoparms.begin(); it != repoparms.end(); ++it)
 	{
 		if (!it->second.m_pHooks)
 			continue;
