@@ -314,16 +314,21 @@ string GetStatReport()
 }
 
 
-errnoFmter::errnoFmter()
+errnoFmter::errnoFmter(const char *prefix)
 {
 	char buf[32];
 	buf[0]=buf[31]=0x0;
 
+	if(prefix)
+		assign(prefix);
+
+	int err=errno;
+
 #ifdef _GNU_SOURCE
 //#warning COMPILER BUG DETECTED -- _GNU_SOURCE was preset in C++ mode
-	assign(strerror_r(errno, buf, sizeof(buf)-1));
+	append(strerror_r(err, buf, sizeof(buf)-1));
 #else
-	assign(strerror_r(errno, buf, sizeof(buf)-1) ? "UNKNOWN ERROR" : buf);
+	append(strerror_r(errno, buf, sizeof(buf)-1) ? "UNKNOWN ERROR" : buf);
 #endif
 }
 
