@@ -50,7 +50,7 @@ bool CAddrInfo::Resolve(const string & sHostname, const string &sPort,
 		return false;
 	}
 
-	LOG("Host resolved: " << m_resolvedInfo->ai_canonname);
+	LOG("Host resolved");
 
 	for (struct addrinfo * pCur=m_resolvedInfo; pCur; pCur = pCur->ai_next)
 	{
@@ -60,12 +60,14 @@ bool CAddrInfo::Resolve(const string & sHostname, const string &sPort,
 			return true;
 		}
 	}
+	LOG("couldn't find working DNS config");
+
 	sErrorBuf="500 DNS resolution error";
 
 	// TODO: remove me from the map
 	return false;
 }
-;
+
 CAddrInfo::~CAddrInfo()
 {
 	if (m_resolvedInfo)
