@@ -6,6 +6,7 @@
 #include "acfg.h"
 #include "dlcon.h"
 
+#include "tcpconnect.h"
 #include "fileitem.h"
 #include <errno.h>
 #include "dljob.h"
@@ -18,7 +19,7 @@ using namespace MYSTD;
 
 
 dlcon::dlcon(bool bManualExecution, string *xff) :
-m_bSingleRun(bManualExecution)
+m_bStopWhenIdle(bManualExecution)
 {
 	LOGSTART("dlcon::dlcon");
 	m_wakepipe[0]=m_wakepipe[1]=-1;
@@ -40,15 +41,15 @@ void dlcon::EnqJob(tDlJob *todo)
 	if (m_wakepipe[1]>=0)
 		POKE(m_wakepipe[1]);
 }
+
 void dlcon::AddJob(tFileItemPtr m_pItem, 
-		acfg::tHostiVec *pBackends, const MYSTD::string & sPatSuffix)
+		acfg::tRepoData *pBackends, const MYSTD::string & sPatSuffix)
 {
-	//lockguard g(m_pItem.get());
 	EnqJob(new tDlJob(this, m_pItem, pBackends, sPatSuffix));
 }
+
 void dlcon::AddJob(tFileItemPtr m_pItem, tHttpUrl hi)
 {
-	//lockguard g(m_pItem.get());
 	EnqJob(new tDlJob(this, m_pItem, hi.sHost, hi.sPath));
 }
 
@@ -56,9 +57,9 @@ void dlcon::SignalStop()
 {
 	setLockGuard;
 	
-	// stop all activity as far as possible
-	m_bSingleRun=true;
-	
+	// stop all activity as soon as possible
+	m_bStopWhenIdle=true;
+
 	/* forget all but the first, those has to be fetched by the own downloader. If first
 	 * item is being downloaded it will bail out RSN because of getting into NOMOREUSERS state. */
 	while(m_qToReceive.size()>1)
@@ -72,22 +73,18 @@ void dlcon::SignalStop()
 dlcon::~dlcon()
 {
 	LOGSTART("dlcon::~dlcon, Destroying dlcon");
-	
 	checkforceclose(m_wakepipe[0]);
-
 	checkforceclose(m_wakepipe[1]);
-		
 }
-	
+
 void dlcon::WorkLoop()
 {
 	LOGSTART("dlcon::WorkLoop");
-    string sCurrentlyConnectedHost, sErrorMsg;
+    string sErrorMsg;
     tSS sendBuf;
     
     // tmp stuff
-    tDlJob *pj=NULL;
-    int r, nTolErrorCount(0);
+    int nTolErrorCount(0);
 	  
 	if (!m_InBuf.init(acfg::dlbufsize))
 	{
@@ -104,42 +101,76 @@ void dlcon::WorkLoop()
 	fd_set rfds, wfds;
 	struct timeval tv;
 	       
-	bool bStopRequesting=false; 
-  
-	while (true)
+	bool bStopAddingRequests=false;
+
+	// keep related state vars together
+	struct
+	{
+		tTcpHandlePtr ptr;
+		int state; // negative = not connected, otherwise count of requests in the pipeline
+		int fd;
+		inline void reset()
+		{
+			ptr.reset();
+			state=fd=-1;
+		}
+		inline void recycle()
+		{
+			tcpconnect::RecycleIdleConnection(ptr);
+			state = fd = -1;
+		}
+	} con;
+	con.reset();
+
+	next_dlcon_cycle:
+	for(;;)
 	{
 		int nMaxFd=m_wakepipe[0];
 		FD_ZERO(&rfds);
 		FD_ZERO(&wfds);
         FD_SET(m_wakepipe[0], &rfds);
-		
+        tDlJob *pCurJob=NULL;
+
 		{
+        	// needs to protect against concurent queue modification
 			setLockGuard;
 			
-			pj=NULL;
-			
 			dbgline;
 			if(m_qToReceive.empty())
 			{
-				if(m_bSingleRun)
+				dbgline;
+				// if we wait then it might time out, but others may make use of it ATM
+				if(0 == con.state)
+					con.recycle();
+
+				// received a signal to stop the work as soon as the process is completed?
+				if(m_bStopWhenIdle)
 					return;
 				
 				dbgline;
-				goto do_select; // wakepipe is in the fds set, just wait for parent there
+
+				// and just wait for parent notification there
+				goto ready_for_select;
 			}
 
-			pj = m_qToReceive.front();
+			pCurJob = m_qToReceive.front();
 
-
-			if(pj->bSuggestReconnect)
+			if(pCurJob->bSuggestReconnect)
 			{
+				pCurJob->bSuggestReconnect=false;
 				ldbg("found host change flag, prepare reconnection");
-				pj->bSuggestReconnect=false;
+				if(0 == con.state)
+					con.recycle();
+				else
+					con.reset();
 				nTolErrorCount=0; // new host, new luck...
-				sCurrentlyConnectedHost.clear();
-				dbgline;
 			}
-			if(sCurrentlyConnectedHost.empty())
+
+			/* fresh state and/or going for reconnection. Pick up a working hostname and
+		    * connect to it.
+		    * */
+
+			if(con.state<0)
 			{
 				dbgline;
 				// set this right now... no matter how the rest looks, current
@@ -147,108 +178,116 @@ void dlcon::WorkLoop()
 				//
 				m_InBuf.clear();
 				sendBuf.clear();
-				bStopRequesting=false;
+				bStopAddingRequests=false;
 				// just be sure about that
-				_Disconnect();
-								
-				/* fresh state and/or going for reconnection. Pick up a working hostname and
-			    * connect to it.
-			    * */
-    
-				if(!pj->FindConfig()) // bad, no usable host in the first item...
+				con.reset();
+
+				// bring job queue into a fresh state
+				for(dljIter it=m_qToReceive.begin(); it!=m_qToReceive.end(); )
 				{
-					pj->MarkFailed(sErrorMsg.empty() ? "503 No usable download host found" : sErrorMsg);
+					tDlJob *p=*it;
+					p->bSuggestReconnect=p->bRequestEnqueued=false;
 					dbgline;
-					
-					delete pj;
+					if(p->HasBrokenStorage())
+					{
+						if(p == pCurJob)
+							pCurJob=NULL;
+						delete p;
+						m_qToReceive.erase(it++);
+						continue;
+					}
+					++it;
+				}
+				if(!pCurJob) // lost front job, check the state
+					continue;
+
+				if(!pCurJob->FindConfig()) // bad, no usable host in the first item...
+				{
+					pCurJob->UnregDownloader(sErrorMsg.empty()
+							? "503 No usable download host found"
+									: sErrorMsg);
+					delete pCurJob;
 					m_qToReceive.pop_front();
 					continue;
 				}
 				
-				const string sTargetHost = pj->GetPeerName();
+				// can do internal work without locking queue (against modifications)
+				__lockguard.unLock();
 
-				acfg::tHostiVec::IHookHandler *pobs = pj->GetConnStateObserver();
+				const string sTargetHost = pCurJob->GetPeerName();
+				ldbg("new target host: "<<sTargetHost);
+
+
+				acfg::tRepoData::IHookHandler *pobs = pCurJob->GetConnStateObserver();
 				if(pobs)
 					pobs->JobConnect();
 
-				__lockguard.unLock();
-				// might override it with proxy stuff
-				bool bOk=_Connect(sTargetHost, sErrorMsg);
+				con.ptr = tcpconnect::CreateConnected(sTargetHost, sErrorMsg);
 
-				ldbg("Connection ok? "<<bOk<<", remember host? "<<sTargetHost);
+				ldbg("Connection state: "<<(con.ptr?"ok":"failed"));
 
-				m_pConnStateObserver=pobs;
+				if(con.ptr)
+				{
+	                dbgline;
+					con.ptr->SetStateObserver(pobs);
+					con.state = 0;
+					con.fd = con.ptr->GetFD();
+				}
+				else
+					pCurJob->BlacklistBackend();
 
-                if(bOk)
-					sCurrentlyConnectedHost=sTargetHost;
-				
-                dbgline;
-                
+                // queue modification possible again
 				__lockguard.reLock();
 				dbgline;
 				
-                	
-				if(!bOk)
-					pj->BlacklistBackend();
-				
-				// weed out those with no way out, either in error state already, or 
-				// ran out of backends (and notify users in this case)
-				dljIter it=m_qToReceive.begin();
-				bool bPrimaryFailed=false;
-				while(it!=m_qToReceive.end())
+				// recheck the queue, drop those without any possible download strategy
+				for(dljIter it=m_qToReceive.begin(); it!=m_qToReceive.end(); )
 				{
 					tDlJob *p=*it;
-					p->bSuggestReconnect=p->bRequestPrepared=false;
-					dbgline;
-					
-					bool bDropIt=p->HasBrokenStorage();
-					if(!bDropIt)
+					if(!p->FindConfig()) // oh no, no more potential mirrors. Must tell the user.
 					{
-						dbgline;
-						
-						if( ! p->FindConfig())
-						{
-							dbgline;
-							bDropIt=true;
-							p->MarkFailed(sErrorMsg); // sensible only for the first... whatever
-						}
-					}
-					// ok, drop it for sure
-					if(bDropIt)
-					{
-						if(pj == p)
-							bPrimaryFailed=true;
-						
-						delete *it;
-						dljIter itmp=it++;
-						m_qToReceive.erase(itmp);
-						
-						dbgline;
+						p->UnregDownloader(sErrorMsg);
+						if(pCurJob == p)
+							pCurJob = NULL;
+						delete p;
+						m_qToReceive.erase(it++);
 						continue;
 					}
-					it++;
+					++it;
 				}
-				if(bPrimaryFailed)
+				// ooops, lost current job?
+				if(!pCurJob)
 				{
-					ldbg("Resetting sCurrentlyConnectedHost, primary source failed");
-					sCurrentlyConnectedHost.clear();
+					// can shift the pointer to the next if the connection is usable?
+					if(con.ptr && !m_qToReceive.empty()
+							&& m_qToReceive.front()->GetPeerName() == con.ptr->GetHostname())
+					{
+						pCurJob=m_qToReceive.front();
+					}
+					else // this was a fresh connection, reuse it
+						con.recycle();
+					continue;
 				}
 				
-				for(it=m_qToReceive.begin(); it!=m_qToReceive.end(); it++)
+				if(!con.ptr) // failed above? will recheck the connection state
+					continue;
+
+				for(dljIter it=m_qToReceive.begin(); it!=m_qToReceive.end(); it++)
 				{
 					dbgline;
-					if(! (*it)->AppendRequest(sTargetHost, sendBuf))
+					tDlJob *p=*it;
+					if(p->AppendRequest(sTargetHost, sendBuf, con.ptr->GetProxyData()))
+						con.state++;
+					else
 					{
-						// refused -> needs to reconnect later
-						(*it)->bSuggestReconnect=true;
-						bStopRequesting=true;  // MUST set this
+						// refused? -> needs to reconnect later
+						p->bSuggestReconnect=true;
+						bStopAddingRequests=true;  // MUST set this
 						dbgline;
 						break;
 					}
 				}
-				
-				if(!bOk) // failed somewhere above, recheck
-					continue;
+
 			} // (re)connect done
 
 			dbgline;
@@ -256,107 +295,97 @@ void dlcon::WorkLoop()
           
 		} // end of locked section	
 		
-					
-		if(m_conFd>=0)
+		ready_for_select:
+
+		if(con.fd>=0)
 		{
-			FD_SET(m_conFd, &rfds);
-			nMaxFd=std::max(m_conFd, nMaxFd);
+			FD_SET(con.fd, &rfds);
+			nMaxFd=std::max(con.fd, nMaxFd);
 			
 			if(sendBuf.size())
 			{
 				ldbg("Needs to send " << sendBuf.size() << " bytes")
-				FD_SET(m_conFd, &wfds);
-				nMaxFd=std::max(m_conFd, nMaxFd);
+				FD_SET(con.fd, &wfds);
+				nMaxFd=std::max(con.fd, nMaxFd);
 			}
 		}
 
-		do_select:
-		
 		ldbg("select dlcon");
 		tv.tv_sec = acfg::nettimeout;
 		tv.tv_usec = 0;
 		
-		r=select(nMaxFd+1, &rfds, &wfds, NULL, &tv);
+		int r=select(nMaxFd+1, &rfds, &wfds, NULL, &tv);
 		
 		if (r<0)
 		{
 			if (EINTR == errno)
 				continue;
 
-			aclog::errnoFmter fer;
-			ldbg("FAILURE: select, " << fer);
+			aclog::errnoFmter fer("FAILURE: select, ");
+			LOG(fer);
 		    sErrorMsg=string("500 Internal malfunction, ")+fer;
 		    goto drop_and_restart_stream;
 		}
-		else if(r==0)
+		else if(r==0) // must be a timeout
 		{
-			if(!sCurrentlyConnectedHost.empty()) 
-            {
-				// catch errno ASAP
-				aclog::errnoFmter fer;
-                ldbg("Select timeout for sCurrentlyConnectedHost:" << sCurrentlyConnectedHost << ". Trigger reconnection..., m_sConnectedHost.clear()");
-                sCurrentlyConnectedHost.clear();
-                
-                if(pj && pj->HasStarted())
-                {
-                	aclog::err("Warning, disconnected during package download");
-                	sErrorMsg=string("500 Connection abort, ")+fer;
-                	goto drop_and_restart_stream;
-                }
-            }
-			continue;
+			if(con.state>0)
+			{
+				sErrorMsg=aclog::errnoFmter("500 Connection abort, ");
+				aclog::err("Warning, disconnected during package download");
+			}
+			goto drop_and_restart_stream;
 		}
 
 		if (FD_ISSET(m_wakepipe[0], &rfds))
 		{
+			dbgline;
 			for(int tmp; read(m_wakepipe[0], &tmp, 1) > 0; ) ;
 			
 			setLockGuard;
 			// got new stuff? and needs to prepare requests? and queue is not empty?
 			// walk around and send requests for them, if possible
-			if(	!bStopRequesting &&
-				!sCurrentlyConnectedHost.empty() &&
-				!m_qToReceive.empty())
+			if(	!bStopAddingRequests && con.ptr && !m_qToReceive.empty())
 			{
 				ldbg("Preparing requests for new stuff...");
 				
-				// walks backward from the end to find a good position where previous request-sending stoped,
-				// instead of just starting from the beginning.
-				
-				// WARNING!!! Optimistic code, relies on condition checks above
+				// walks backward from the end to find a good position where previous
+				// request-sending was interrupted instead of just starting from the beginning.
 				
 				dljIter it=m_qToReceive.end();
 				for(it--; it!=m_qToReceive.begin(); it--)
-					if( (*it)->bRequestPrepared )
+					if( (*it)->bRequestEnqueued )
 						break;
 				
 				for(; it!=m_qToReceive.end(); it++)
 				{
 					tDlJob *pjn=*it;
 					// one position to far in backwalk
-					if(pjn->bRequestPrepared)
+					if(pjn->bRequestEnqueued)
 						continue;
 					
 					// found the first unseen, do stuff
 					
-					pjn->bRequestPrepared=true;
-					if(!pjn->FindConfig() || !pjn->AppendRequest(sCurrentlyConnectedHost, sendBuf)) 
+					pjn->bRequestEnqueued=true;
+					if(!pjn->FindConfig() || !pjn->AppendRequest(con.ptr->GetHostname(),
+							sendBuf, con.ptr->GetProxyData()))
 					{
-						// bad stuff, check later
-						bStopRequesting=true;
+						// bad stuff, to be reported later
+						bStopAddingRequests=true;
 						pjn->bSuggestReconnect=true;
 						break;
 					}
+					else
+						con.state++;
 				}
 			}
 		}
 
-		if (m_conFd>=0 && sendBuf.size() && FD_ISSET(m_conFd, &wfds))
+		if (con.fd>=0 && sendBuf.size() && FD_ISSET(con.fd, &wfds))
 		{
-			FD_CLR(m_conFd, &wfds);
-			
+			FD_CLR(con.fd, &wfds);
+
 			ldbg("Sending data...\n" << sendBuf);
-			int s=::send(m_conFd, sendBuf.data(), sendBuf.length(), MSG_NOSIGNAL);
+			int s=::send(con.fd, sendBuf.data(), sendBuf.length(), MSG_NOSIGNAL);
 			ldbg("Sent " << s << " bytes from " << sendBuf.length());
 			if (s<0)
 			{
@@ -371,26 +400,23 @@ void dlcon::WorkLoop()
                 sendBuf.drop(s);
 		}
 	    
-		if (m_conFd>=0 && FD_ISSET(m_conFd, &rfds))
+		if (con.fd>=0 && FD_ISSET(con.fd, &rfds))
 		{
-			if(!pj) // huh, sends something without request? Maybe disconnect hang (zero read)?
+			if(!pCurJob) // huh, sends something without request? Maybe disconnect hang (zero read)?
 			{
 				ldbg("BUG: not expected data");
 				// just disconnect, and maybe reconnect
-				sCurrentlyConnectedHost.clear();
+				con.reset();
 				continue;
 			}
-			int r = m_InBuf.sysread(m_conFd);
+			int r = m_InBuf.sysread(con.fd);
 			if (r <= 0)
 			{
-				if(r==-EAGAIN)
-					continue; // should never happen, though.
+				// pickup the error code for later and kill current connection ASAP
+				sErrorMsg=aclog::errnoFmter("502 ");
+				con.reset();
 				
-				// pickup the error code and make sure it's closed ASAP
-				aclog::errnoFmter f;
-				_Disconnect();
-				
-				if(pj)
+				if(pCurJob)
 				{
 					/*
 					 * disconnected? Maybe because of end of Keep-Alive sequence, or
@@ -401,35 +427,34 @@ void dlcon::WorkLoop()
 					if(++nTolErrorCount < MAX_RETRIES)
 					{
 						
-						if( ! pj->HasStarted())
+						if( ! pCurJob->HasStarted())
 						{
-							ldbg("MAX_RETRIES not reached, resetting sCurrentlyConnectedHost");
-							sCurrentlyConnectedHost.clear();
+							ldbg("MAX_RETRIES not reached, just reconnect");
+							con.reset();
 							continue;
 						}
 					}
 					else
 					{   // too many retries for this host :-(
-						pj->BlacklistBackend();
+						pCurJob->BlacklistBackend();
 						nTolErrorCount=0;
 					}
 				}
 				
-				sErrorMsg=string("502 ")+f;
                 goto drop_and_restart_stream;
 			}
         }
 
-		while( ! m_InBuf.empty() && pj)
+		while( ! m_InBuf.empty() && pCurJob)
 		{
-			ldbg("Processing input of " << pj->RemoteUri() );
-			tDlJob::tDlResult res=pj->ProcessIncomming(m_InBuf, sErrorMsg);
-			ldbg("... input processing result: " << res);
+			ldbg("Processing job for " << pCurJob->RemoteUri() );
+			tDlJob::tDlResult res=pCurJob->ProcessIncomming(m_InBuf, sErrorMsg);
+			ldbg("... incomming data processing result: " << res);
 			switch(res)
 			{
 				case(tDlJob::R_MOREINPUT):
 				case(tDlJob::R_NOOP):
-					goto leave_recv_loop; // will get more in the next loop
+					goto next_dlcon_cycle; // will get more in the next loop
 				case(tDlJob::R_SKIPITEM): // will restart the stream
 				// item was ours but download aborted... might do more fine recovery in the future
 				case(tDlJob::R_ERROR_LOCAL):
@@ -440,33 +465,33 @@ void dlcon::WorkLoop()
 				case(tDlJob::R_DONE):
 				{
 					nTolErrorCount=0; // well done and can sleep now, should be resumed carefully later
+					con.state--;
 					
 					setLockGuard;
-					delete pj;
+					delete pCurJob;
 					m_qToReceive.pop_front();
-					pj = m_qToReceive.empty() ? NULL : m_qToReceive.front();
+					pCurJob = m_qToReceive.empty() ? NULL : m_qToReceive.front();
 					ldbg("Remaining dlitems: " << m_qToReceive.size());
 					continue;
 				}
 				break;
 			}
         }
-		leave_recv_loop:
 		
 		continue;
 		
 		drop_and_restart_stream:
-		if(pj)
+
+		ldbg("Resetting pCurJob, resetting connection");
+		con.reset();
+		if(pCurJob)
 		{
-			ldbg("Resetting pj, resetting sCurrentlyConnectedHost");
-			pj->MarkFailed(sErrorMsg);
-			sCurrentlyConnectedHost.clear();
+			pCurJob->UnregDownloader(sErrorMsg);
 			setLockGuard;
-			delete pj;
+			delete pCurJob;
 			m_qToReceive.pop_front();
-			pj=NULL;
+			pCurJob=NULL;
 			continue;
 		}
 	}
 }
-
