@@ -24,6 +24,8 @@
 #include <iostream>
 #include <algorithm>
 
+#include <unistd.h>
+
 using namespace MYSTD;
 
 #define ENABLED
@@ -42,7 +44,7 @@ tCacheMan::tCacheMan(int fd) :
 	tWuiBgTask(fd), m_nTimeNow(0),
 	m_bErrAbort(false), m_bVerbose(false), m_bForceDownload(false),
 	m_bScanInternals(false), m_bByPath(false), m_bByChecksum(false), m_bSkipHeaderChecks(false),
-	m_bNeedsStrictPathsHere(false),
+	m_bNeedsStrictPathsHere(false), m_bTruncateDamaged(false),
 	m_nErrorCount(0),
 	m_nProgIdx(0), m_nProgTell(1), m_pDlcon(NULL)
 {
@@ -147,7 +149,7 @@ bool tCacheMan::RefetchFile(tFileItemPtr fi, cmstring &sFilePathRel, mstring &sE
 	// abusing url class to extract base directory
 
 	tHttpUrl url;
-	acfg::tHostiVec *pBackends(NULL);
+	acfg::tRepoData *pBackends(NULL);
 
 	if (!url.SetHttpUrl(sFilePathRel))
 	{
@@ -222,7 +224,9 @@ bool tCacheMan::DownloadIdxFile(const MYSTD::string & sFilePathRel, string &sErr
 	FiStatus fistate=FIST_FRESH;
 	tFileItemPtr fi;
 
-	//bool nix=StrHas(sFilePathRel, "debrep/dists/experimental/main/source/Sources.diff/Index");
+#ifdef DEBUG
+	bool nix=StrHas(sFilePathRel, "etch-unikl");
+#endif
 
 	const tIfileAttribs &flags=GetFlags(sFilePathRel);
 	if(flags.uptodate)
@@ -273,7 +277,11 @@ bool tCacheMan::DownloadIdxFile(const MYSTD::string & sFilePathRel, string &sErr
 			SendFmt() << "<i>(" << fi->GetTransferCount() / 1024 << "KiB)</i>\n";
 		}
 		else
+		{
 			bSuccess = flags.forgiveDlErrors;
+			if(bSuccess)
+				SendFmt() << "<i>(ignored)</i>\n";
+		}
 	}
 #ifdef DEBUG
 	else
@@ -351,7 +359,7 @@ bool tCacheMan::DownloadSimple(const MYSTD::string & sFilePathRel)
 }
 
 
-static const string relKey("/Release");
+static const string relKey("/Release"), inRelKey("/InRelease");
 
 #define ERRMSGABORT if(m_nErrorCount && m_bErrAbort) { SendChunk(sErr); return; }
 #define ERRABORT if(m_nErrorCount && m_bErrAbort) { return; }
@@ -650,21 +658,27 @@ bool tCacheMan::GetAndCheckHead(cmstring & sDataFileRel, cmstring &sReferencePat
 
 bool tCacheMan::Inject(cmstring &from, cmstring &to)
 {
+	LOGSTART("tCacheMan::Inject");
+
 	// XXX should it really filter it here?
 	if(GetFlags(to).uptodate)
 		return true;
 
 #ifdef DEBUG
+	bool nix = stmiss!=from.find("debian/dists/lenny/contrib/binary-amd64/Packages");
 	SendFmt()<<"Replacing "<<to<<" with " << from <<  "<br>\n";
 #endif
 
 	header h;
 	filereader data;
 
-	if(h.LoadFromFile(SABSPATH(from+".head")) <= 0 || !data.OpenFile(SABSPATH(from), true))
+	if(h.LoadFromFile(SABSPATH(from+".head")) <= 0
+			|| !data.OpenFile(SABSPATH(from), true)
+			|| ! h.h[header::CONTENT_LENGTH]
+			|| off_t(data.GetSize()) != atoofft(h.h[header::CONTENT_LENGTH]))
 	{
 #ifdef DEBUG
-	SendFmt()<<"Cannot read "<<from<<".head<br>\n";
+	SendFmt()<<"Cannot read "<<from<<".head or bad data<br>\n";
 #endif
 		return false;
 	}
@@ -673,22 +687,26 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to)
 	if (!fi)
 		return false;
 
-	bool bRes=(fi->Setup(true) != FIST_ERROR
-			&& fi->DownloadStartedStoreHeader(h, NULL)
-			&& fi->StoreFileData(data.GetBuffer(), data.GetSize())
-			&& fi->StoreFileData(NULL, 0)
-			&& fi->GetStatus() == FIST_COMPLETE);
+	bool bError = ( fi->Setup(true) == FIST_ERROR
+			|| ! fi->DownloadStartedStoreHeader(h, NULL)
+			|| ! fi->StoreFileData(data.GetBuffer(), data.GetSize())
+			|| ! fi->StoreFileData(NULL, 0)
+			|| fi->GetStatus() != FIST_COMPLETE);
 
 	UpdateFingerprint(to, -1, NULL, NULL);
 
 	fi->Unreg();
 
 #ifdef DEBUG
-	if(!bRes) SendChunk("<font color=red>Inject failed</font><br>");
+	if(bError)
+	{
+		LOG("Houston, we have a problem");
+		SendFmt() << "<font color=red>Inject failed</font><br>";
+	}
 #endif
 	tIfileAttribs &atts = SetFlags(to);
-	atts.uptodate=atts.vfile_ondisk=bRes;
-	return bRes;
+	atts.uptodate=atts.vfile_ondisk=!bError;
+	return !bError;
 }
 
 
@@ -773,6 +791,10 @@ bool tCacheMan::Propagate(const string &donor, tContId2eqClass::iterator eqClass
 #ifdef DEBUG
 				SendFmt() << "Defused bro: " << sBasename<<*ps<<"<br>\n";
 #endif
+				// also, we don't care if the pdiff index vanished for some reason
+				kv = m_indexFilesRel.find(sBasename+".diff/Index");
+				if(kv!=m_indexFilesRel.end())
+					kv->second.forgiveDlErrors=true;
 			}
 		}
 	}
@@ -863,7 +885,7 @@ void tCacheMan::UpdateIndexFiles()
 	{
 		const string &sPathRel=it->first;
 
-		if(!endsWith(sPathRel, relKey))
+		if(!endsWith(sPathRel, relKey) && !endsWith(sPathRel, inRelKey))
 			continue;
 
 		if(!DownloadIdxFile(sPathRel, sErr))
@@ -877,6 +899,11 @@ void tCacheMan::UpdateIndexFiles()
 
 			continue;
 		}
+
+		// InRelease comes before Release so we can just drop the other one
+		if(endsWith(sPathRel, inRelKey))
+			m_indexFilesRel.erase(sPathRel.substr(0, sPathRel.size()-inRelKey.size())+relKey);
+
 		m_indexFilesRel[sPathRel].uptodate=true;
 
 		releaseStuffReceiver recvr;
@@ -1008,7 +1035,7 @@ void tCacheMan::UpdateIndexFiles()
 		int nProbeCnt(3);
 		string patchidxFileToUse;
 		deque<tPatchEntry> patchList;
-		tFingerprint *pEndSum;
+		tFingerprint *pEndSum(NULL);
 		tPListConstIt itPatchStart;
 
 		if(CheckAbortCondition())
@@ -1120,10 +1147,11 @@ void tCacheMan::UpdateIndexFiles()
 							return;
 
 						if (m_bVerbose)
-							SendFmt() << "Found patching base candidate, unpacked to " << sPatchBaseAbs << "<br>";
+							SendFmt() << "Found patching base candidate, unpacked to "
+							<< sPatchBaseAbs << "<br>";
 
-						if (PatchFile(sPatchBaseAbs, patchidxFileToUse, itPatchStart, patchList.end(),
-								pEndSum))
+						if (PatchFile(sPatchBaseAbs, patchidxFileToUse, itPatchStart,
+								patchList.end(), pEndSum))
 						{
 
 							if(CheckAbortCondition())
@@ -1316,7 +1344,7 @@ tCacheMan::enumIndexType tCacheMan::GuessIndexType(const mstring &sPath)
 	if (sPureIfileName == "Sources")
 		return EIDX_SOURCES;
 
-	if (sPureIfileName == "Release")
+	if (sPureIfileName == "Release" || sPureIfileName=="InRelease")
 		return EIDX_RELEASE;
 
 	if (sPureIfileName == "Index")
@@ -1577,15 +1605,17 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 		idxType = EIDX_DEBSRCLIKE;
 		goto REDO_AS_TYPE;
 
-		case EIDX_SOURCES:
-			info.fpr.csType = CSTYPE_MD5;
-			sStartMark="Files:";
-			idxType = EIDX_DEBSRCLIKE;
-			goto REDO_AS_TYPE;
+	case EIDX_SOURCES:
+		info.fpr.csType = CSTYPE_MD5;
+		sStartMark="Files:";
+		idxType = EIDX_DEBSRCLIKE;
+		goto REDO_AS_TYPE;
+
 	case EIDX_RELEASE:
-			info.fpr.csType = CSTYPE_MD5;
-			sStartMark="MD5Sum:";
-			// fall-through, parser follows
+		info.fpr.csType = CSTYPE_SHA1;
+		sStartMark="SHA1:";
+		// fall-through, parser follows
+
 	case EIDX_DEBSRCLIKE:
 		for (bool bNoEof=true; bNoEof;)
 		{
@@ -1648,7 +1678,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 					ret.HandlePkgEntry(info, false);
 					break;
 				default:
-					ASSERT(!"Originally determined type cannot lead into this case!");
+					ASSERT(!"Originally determined type cannot reach this case!");
 					break;
 				}
 			}
@@ -1856,8 +1886,16 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 			{
 				if ( ! (fprHave == entry.fpr))
 				{
-					SendFmt()<<"<font color=red>BAD: "<<sPathAbs<<"</font><br>\n";
-					goto keep_in_trash;
+					if(m_bTruncateDamaged)
+					{
+						SendFmt()<<"<font color=orange>BAD: "<<sPathAbs<<" (truncating)</font><br>\n";
+						::truncate(sPathAbs.c_str(), 0);
+					}
+					else
+					{
+						SendFmt()<<"<font color=red>BAD: "<<sPathAbs<<"</font><br>\n";
+						goto keep_in_trash;
+					}
 				}
 			}
 		}
@@ -1998,6 +2036,7 @@ void tCacheMan::SetCommonUserFlags(cmstring &cmd)
 	m_bVerbose=(cmd.find("beVerbose")!=stmiss);
 	m_bForceDownload=(cmd.find("forceRedownload")!=stmiss);
 	m_bSkipHeaderChecks=(cmd.find("skipHeadChecks")!=stmiss);
+	m_bTruncateDamaged=(cmd.find("truncNow")!=stmiss);
 }
 
 void expiration::Action(const string & cmd)
@@ -2176,6 +2215,7 @@ bool expiration::ProcessRegular(const string & sPath, const struct stat &stinfo)
 	else
 	{
 		AddIFileCandidate(sDirRel+sBasename);
+#warning for d-i images, also add their MD5SUMS file, always
 		//SendChunk(string("<br>hm?<br>")+sDirnameAndSlash + " -- " + sBasename);
 		tDiskFileInfo &finfo = m_trashCandSet[tFileNdir(sBasename, sDirRel)];
 		finfo.nLostAt = m_nTimeNow;
