@@ -158,6 +158,8 @@ void fileitem::ResetCacheState()
 
 FiStatus fileitem::Setup(bool bCheckFreshness)
 {
+	LOGSTART2("fileitem::Setup", bCheckFreshness);
+
 	setLockGuard;
 
 	if(status>FIST_FRESH)
@@ -170,6 +172,8 @@ FiStatus fileitem::Setup(bool bCheckFreshness)
 		if(200 != m_head.getStatus())
 			goto error_clean;
 		
+		LOG("good head");
+
 		m_nSizeSeen=GetFileSize(m_sPath, 0);
 
 		// some plausibility checks
@@ -178,6 +182,7 @@ FiStatus fileitem::Setup(bool bCheckFreshness)
 			const char *p=m_head.h[header::LAST_MODIFIED];
 			if(!p)
 				goto error_clean; // suspicious, cannot use it
+			LOG("check freshness, last modified: " << p );
 		}
 		else
 		{
@@ -191,6 +196,8 @@ FiStatus fileitem::Setup(bool bCheckFreshness)
 				if(nContLen < m_nSizeSeen)
 					goto error_clean;
 				
+				LOG("Content-Length has a sane range");
+
 				// is it complete? and 0 value also looks weird, try to verify
 				if(m_nSizeSeen == nContLen && nContLen>0)
 				{
@@ -212,6 +219,7 @@ FiStatus fileitem::Setup(bool bCheckFreshness)
 		if(!bCheckFreshness)
 			m_nSizeSeen=GetFileSize(m_sPath, 0);
 	}
+	LOG("resulting status: " << status);
 	return status;
 
 	error_clean:
@@ -407,8 +415,14 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		}
 		
 		if (m_filefd<0)
-			BOUNCE( (errno==ENOSPC ? "503 OUT OF DISK SPACE" :
-			"503 Cache storage error while opening data file"));
+		{
+			aclog::errnoFmter efmt("503 Cache storage error - ");
+#ifdef DEBUG
+			BOUNCE((efmt+m_sPath).c_str());
+#else
+			BOUNCE(efmt.c_str());
+#endif
+		}
 		
 		if(0!=fstat(m_filefd, &stbuf) || !S_ISREG(stbuf.st_mode))
 			SETERRORKILLFILE("503 Not a regular file");
@@ -416,7 +430,9 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		// crop, but only if the new size is smaller. MUST NEVER become larger (would fill with zeros)
 		if(m_nSizeChecked <= m_nSizeSeen)
 		{
-			if(0!=ftruncate(m_filefd, m_nSizeChecked))
+			if(0==ftruncate(m_filefd, m_nSizeChecked))
+				fdatasync(m_filefd);
+			else
 				SETERRORKILLFILE("503 Cannot change file size");
 		}
 		else if(m_nSizeChecked>m_nSizeSeen) // should never happen and caught by the checks above
@@ -465,10 +481,13 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 	return false;
 }
 
-bool fileitem::StoreFileData(const char *data, unsigned int size) {
-	
+bool fileitem::StoreFileData(const char *data, unsigned int size)
+{
+
 	setLockGuard;
 
+	LOGSTART2("fileitem::StoreFileData", "status: " << status << ", size: " << size);
+
 	// something might care, most likely... also about BOUNCE action
 	notifyAll();
 	
@@ -479,6 +498,12 @@ bool fileitem::StoreFileData(const char *data, unsigned int size) {
 	
 	if (size==0)
 	{
+		if(FIST_COMPLETE == status)
+		{
+			LOG("already completed");
+			return true;
+		}
+
 		status=FIST_COMPLETE;
 
 		if(acfg::debug>1)
@@ -507,9 +532,8 @@ bool fileitem::StoreFileData(const char *data, unsigned int size) {
 				{
 					if(EINTR==errno || EAGAIN==errno)
 						continue;
-					if(ENOSPC==errno)
-						BOUNCE("503 OUT OF DISK SPACE");
-					BOUNCE("503 General storage error");
+					aclog::errnoFmter efmt("503 ");
+					BOUNCE(efmt.c_str());
 				}
 				m_nSizeChecked+=r;
 				size-=r;
