@@ -126,10 +126,10 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 	
 	if(statbuf.st_size>0)
 	{
-		m_nBufSize = statbuf.st_size;
-		m_szFileBuf = (char*) mmap(0, m_nBufSize, PROT_READ, MAP_SHARED, m_fd, 0);
+		m_szFileBuf = (char*) mmap(0, statbuf.st_size, PROT_READ, MAP_SHARED, m_fd, 0);
 		if(m_szFileBuf==MAP_FAILED)
 			goto ofail;
+		m_nBufSize = statbuf.st_size;
 	}
 	else
 	{
@@ -192,6 +192,7 @@ void filereader::Close()
 		deflateEnd(&m_zStream);
 #endif
 	flags=0;
+	m_nBufSize=0;
 }
 
 filereader::~filereader() {
@@ -281,7 +282,7 @@ inline void filereader::_UncompressMoreData() {
 	// work with uncompressed buffer/window...
 	m_UncompBuf.move(); // get unused space if possible
 		
-	if(	flags&FLAG_COMPEOF // cannot uncompress more
+	if(	(flags&FLAG_COMPEOF) // cannot uncompress more
 		|| m_UncompBuf.freecapa()==0 )
 		return;
 	
