@@ -261,6 +261,8 @@ void header::set(eHeadPos key, off_t nValue)
     set(key, buf);
 }
 
+#ifndef MINIBUILD
+
 string header::ToString()
 {
 
@@ -325,6 +327,8 @@ int header::StoreToFile(const string &sPath)
 	return nByteCount;
 }
 
+#endif // MINIBUILD
+
 MYSTD::string header::GenInfoHeaders()
 {
 	    char buf[26];
