@@ -479,7 +479,7 @@ void job::PrepareDownload() {
     string sRawUriPath, sPathResdiual;
     tHttpUrl tUrl; // parsed URL
         
-    acfg::tHostiVec * pBackends(NULL); // appropriate backends
+    acfg::tRepoData * pBackends(NULL); // appropriate backends
     const string * psVname(NULL); // virtual name for storage pool, if available
     
     FiStatus fistate(FIST_FRESH);
