@@ -21,7 +21,7 @@
 using namespace MYSTD;
 
 #define SCACHEFILE (CACHE_BASE+"_impkeycache")
-#define FMTSIG "FMT4"
+#define FMTSIG "FMT5"
 
 /*
  * Algorithm:
@@ -79,31 +79,42 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 		if(m_precachedList.find(sPath)!=m_precachedList.end())
 			return true; // have that already, somewhere...
 		
-		// get the most likely requested contents id
-		tFingerprint fpr;
-	/*	if ( (IsIndexDiff(sPath) && fpr.ScanFile(sPath, CSTYPE_SHA1, true))
-				|| (!IsIndexDiff(sPath) && fpr.ScanFile(sPath, CSTYPE_MD5, false)))
-		*/
-		if( fpr.ScanFile(sPath, CSTYPE_MD5, false))
-			SendFmt()<<"<font color=blue>Checked " << sPath<<" (fingerprint created)</font><br>\n";
-		else
+		for (CSTYPES ctp = CSTYPE_MD5; ctp <= CSTYPE_SHA1; ctp=CSTYPES(int(ctp)+1))
 		{
-			SendFmt()<< "<font color=red>Error checking " << sPath << "</font><br>\n";
-			return true;
-		}
-		
-		// add this entry immediately if needed and get its reference
-		tImpFileInfo & node = m_importMap[fpr];
-		if (node.sPath.empty())
-		{ // fresh, just added to the map
-			node.sPath=sPath;
-			node.mtime=stinfo.st_mtime;
-		}
-		else if(node.sPath!=sPath)
-		{
-			SendFmt() << "<font color=orange>Duplicate found, " << sPath
-					<< " vs. " << node.sPath << ", ignoring new entry.</font><br>\n";
-			m_importRest.push_back(make_pair(fpr, tImpFileInfo(sPath, stinfo.st_mtime)));
+
+			// get the most likely requested contents id
+			tFingerprint fpr;
+			/*	if ( (IsIndexDiff(sPath) && fpr.ScanFile(sPath, CSTYPE_SHA1, true))
+			 || (!IsIndexDiff(sPath) && fpr.ScanFile(sPath, CSTYPE_MD5, false)))
+			 */
+			if (!fpr.ScanFile(sPath, ctp, false))
+			{
+				SendFmt() << "<font color=red>Error checking " << sPath << "</font><br>\n";
+				continue;
+			}
+			if (fpr.size < 50)
+			{
+				// ultra small files, looking like garbage (gz'ed empfy file, ...)
+				continue;
+			}
+
+			SendFmt() << "<font color=blue>Checked " << sPath << " (" << (ctp == CSTYPE_MD5
+					? "MD5"
+					: "SHA1") << " fingerprint created)</font><br>\n";
+
+			// add this entry immediately if needed and get its reference
+			tImpFileInfo & node = m_importMap[fpr];
+			if (node.sPath.empty())
+			{ // fresh, just added to the map
+				node.sPath = sPath;
+				node.mtime = stinfo.st_mtime;
+			}
+			else if (node.sPath != sPath)
+			{
+				SendFmt() << "<font color=orange>Duplicate found, " << sPath << " vs. "
+						<< node.sPath << ", ignoring new entry.</font><br>\n";
+				m_importRest.push_back(make_pair(fpr, tImpFileInfo(sPath, stinfo.st_mtime)));
+			}
 		}
 	}
 	else
@@ -311,11 +322,9 @@ void pkgimport::Action(const string &cmd)
 		if(!fList.is_open())
 			continue;
 
-		// not caching sha1-checksums of gzipped index files, less cumbersome and hardly required
-		if(!it->first.csType == CSTYPE_MD5)
-			continue;
 #define ENDL "\n" // don't flush all the time
 		fList << it->first.size << ENDL
+				<< (int)it->first.csType << ENDL
 				<< it->first.GetCsAsString() << ENDL
 				<< it->second.sPath.substr(m_sSrcPath.size()+1) <<ENDL
 				<< it->second.mtime	<<ENDL;
@@ -326,10 +335,8 @@ void pkgimport::Action(const string &cmd)
 	for(tImportList::const_iterator it=m_importRest.begin();
 	it!=m_importRest.end();it++)
 	{
-		// not caching sha1-checksums of gzipped index files, less cumbersome and hardly required
-		if(!it->first.csType == CSTYPE_MD5)
-			continue;
 		fList << it->first.size << ENDL
+				<< (int)it->first.csType << ENDL
 				<< it->first.GetCsAsString() << ENDL
 				<< it->second.sPath.substr(m_sSrcPath.size()+1) <<ENDL
 				<< it->second.mtime	<<ENDL;
@@ -423,15 +430,25 @@ void pkgimport::_LoadKeyCache(const string & sFileName)
 /*	if(m_bVerbose)
 		SendChunk("Loading fingerprints from key cache... \n");
 		*/
+	off_t sz(-1);
+	int csType(CSTYPE_INVALID);
+
 	for(;;)
 	{
 		info.bFileUsed=false;
-		in>>fpr.size;
+
+		in>>sz;
 		std::getline(in, cs); // newline
-		std::getline(in, cs); // next line
-		fpr.Set(cs, CSTYPE_MD5, fpr.size);
+
+		in>>csType;
+		std::getline(in, cs); // newline
+
+		std::getline(in, cs); // checksum line
+		fpr.Set(cs, (CSTYPES)csType, sz);
+
 		std::getline(in, info.sPath);
 		info.sPath.insert(0, m_sSrcPath+SZPATHSEP);
+
 		in>>info.mtime;
 		std::getline(in, cs); // newline
 
