@@ -16,9 +16,11 @@
 #include <signal.h>
 #include "sockio.h"
 
+#include "cleaner.h"
+
 using namespace MYSTD;
 
-tcpconnect::tcpconnect() :	m_conFd(-1), m_pConnStateObserver(NULL)
+tcpconnect::tcpconnect() :	m_conFd(-1), m_nExpTime(0), m_pConnStateObserver(NULL)
 {
 	m_proxy = acfg::proxy_info.sHost.empty() ? NULL : &acfg::proxy_info;
 }
@@ -26,7 +28,7 @@ tcpconnect::tcpconnect() :	m_conFd(-1), m_pConnStateObserver(NULL)
 tcpconnect::~tcpconnect()
 {
 	LOGSTART("tcpconnect::~tcpconnect, terminating outgoing connection class");
-	_Disconnect();
+	Disconnect();
 }
 
 
@@ -92,7 +94,7 @@ static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t ad
 }
 
 
-bool tcpconnect::_Connect(const string & sHostname, string & sErrorMsg)
+bool tcpconnect::Connect(const string & sHostname, string & sErrorMsg)
 {
 	LOGSTART2("tcpconnect::_Connect", "hostname: " << sHostname);
 	CAddrInfo::SPtr dns = m_proxy ?
@@ -114,7 +116,7 @@ bool tcpconnect::_Connect(const string & sHostname, string & sErrorMsg)
 		if (pInfo->ai_socktype != SOCK_STREAM || pInfo->ai_protocol != IPPROTO_TCP)
 			continue;
 
-		_Disconnect();
+		Disconnect();
 
 		m_conFd = ::socket(pInfo->ai_family, pInfo->ai_socktype, pInfo->ai_protocol);
 		if(m_conFd<0)
@@ -133,6 +135,7 @@ bool tcpconnect::_Connect(const string & sHostname, string & sErrorMsg)
 
 		ldbg("connect() ok");
 		set_nb(m_conFd);
+		m_sHostName = sHostname;
 		return true;
 	}
 
@@ -141,11 +144,11 @@ bool tcpconnect::_Connect(const string & sHostname, string & sErrorMsg)
 	sErrorMsg="500 Connection failure: ";
 	sErrorMsg+=fer;
 	ldbg("Force reconnect, con. failure");
-	_Disconnect();
+	Disconnect();
 	return false;
 }
 
-void tcpconnect::_Disconnect()
+void tcpconnect::Disconnect()
 {
 	LOGSTART("tcpconnect::_Disconnect");
 
@@ -155,7 +158,6 @@ void tcpconnect::_Disconnect()
 	m_pConnStateObserver=NULL;
 
 	forceShutdownClose(m_conFd);
-
 }
 
 void termsocket(int fd)
@@ -189,3 +191,76 @@ void termsocket(int fd)
 			break;
 	};
 }
+
+lockable conPoolMx;
+/*
+bool lessByConnName(const tTcpHandlePtr &a, const tTcpHandlePtr &b)
+{
+	return a->GetHostname() < b->GetHostname();
+}
+typedef std::multiset<tTcpHandlePtr, lessByConnName>  tConpoolmap;
+*/
+MYSTD::multimap<cmstring, tTcpHandlePtr> spareConPool;
+
+tTcpHandlePtr tcpconnect::Create(cmstring &sHostnameHint)
+{
+	lockguard __g(conPoolMx);
+	MYSTD::map<cmstring, tTcpHandlePtr>::iterator it=spareConPool.find(sHostnameHint);
+	if(spareConPool.end() == it)
+		return tTcpHandlePtr(new tcpconnect);
+	tTcpHandlePtr ret=it->second;
+	spareConPool.erase(it);
+	return ret;
+}
+
+tTcpHandlePtr tcpconnect::CreateConnected(cmstring &sHostname, mstring &sErrOut)
+{
+	tTcpHandlePtr p = Create(sHostname);
+	if(!p || !p->Connect(sHostname, sErrOut) || p->GetFD()<0) // failed or worthless
+		p.reset();
+	return p;
+}
+
+void tcpconnect::RecycleIdleConnection(SHARED_PTR<tcpconnect> & handle)
+{
+	if(!handle)
+		return;
+
+	time_t now(time(0));
+	lockguard __g(conPoolMx);
+
+	// a DOS?
+	if(spareConPool.size()<50)
+	{
+		handle->m_nExpTime=now+23;
+		spareConPool.insert(make_pair(handle->GetHostname(), handle));
+	}
+
+	handle.reset();
+}
+
+
+time_t tcpconnect::ExpireCache()
+{
+	lockguard __g(conPoolMx);
+	time_t now(time(0)), ret(cleaner::never);
+
+	for(MYSTD::map<cmstring, tTcpHandlePtr>::iterator it=spareConPool.begin();
+			it!=spareConPool.end(); )
+	{
+		if(it->second)
+		{
+			tcpconnect *p=it->second.get();
+			if(p->m_nExpTime<now)
+			{
+				spareConPool.erase(it++);
+				continue;
+			}
+			else
+				ret=min(ret, it->second->m_nExpTime);
+		}
+
+		++it;
+	}
+	return ret;
+}
